@@ -54,10 +54,48 @@ from digitalio import Direction, DigitalInOut
 _SLOW_CLOCK = 100000
 _FAST_CLOCK = 1000000
 
+
 class AVRprog:
     """
     Helper class used to program AVR chips from CircuitPython.
     """
+
+    class Boards:
+        """
+        Some well known board definitions.
+        """
+
+        # pylint: disable=too-few-public-methods
+        ATtiny13a = {
+            "name": "ATtiny13a",
+            "sig": [0x1E, 0x90, 0x07],
+            "flash_size": 1024,
+            "page_size": 32,
+            "fuse_mask": (0xFF, 0xFF, 0x00, 0x03),
+            "clock_speed": 100000,
+        }
+        ATtiny85 = {
+            "name": "ATtiny85",
+            "sig": [0x1E, 0x93, 0x0B],
+            "flash_size": 8192,
+            "page_size": 64,
+            "fuse_mask": (0xFF, 0xFF, 0x07, 0x3F),
+        }
+        ATmega328p = {
+            "name": "ATmega328p",
+            "sig": [0x1E, 0x95, 0x0F],
+            "flash_size": 32768,
+            "page_size": 128,
+            "fuse_mask": (0xFF, 0xFF, 0x07, 0x3F),
+        }
+        ATmega2560 = {
+            "name": "ATmega2560",
+            "sig": [0x1E, 0x98, 0x01],
+            "flash_size": 262144,
+            "page_size": 256,
+            "fuse_mask": (0xFF, 0xFF, 0x07, 0x3F),
+        }
+
     _spi = None
     _rst = None
 
@@ -72,7 +110,6 @@ class AVRprog:
         self._rst.direction = Direction.OUTPUT
         self._rst.value = True
 
-
     def verify_sig(self, chip, verbose=False):
         """
         Verify that the chip is connected properly, responds to commands,
@@ -83,7 +120,7 @@ class AVRprog:
         self.end()
         if verbose:
             print("Found signature: %s" % [hex(i) for i in sig])
-        if sig != chip['sig']:
+        if sig != chip["sig"]:
             return False
         return True
 
@@ -100,20 +137,21 @@ class AVRprog:
             print("Erasing chip....")
         self.erase_chip()
 
-        self.begin()
+        clock_speed = getattr(chip, "clock_speed", _FAST_CLOCK)
+        self.begin(clock=clock_speed)
 
         # create a file state dictionary
-        file_state = {'line': 0, 'ext_addr': 0, 'eof': False}
-        file_state['f'] = open(file_name, 'r')
+        file_state = {"line": 0, "ext_addr": 0, "eof": False}
+        file_state["f"] = open(file_name, "r")
 
-        page_size = chip['page_size']
+        page_size = chip["page_size"]
 
-        for page_addr in range(0, chip['flash_size'], page_size):
+        for page_addr in range(0, chip["flash_size"], page_size):
             if verbose:
                 print("Programming page $%04X..." % page_addr, end="")
             page_buffer = bytearray(page_size)
             for b in range(page_size):
-                page_buffer[b] = 0xFF     # make an empty page
+                page_buffer[b] = 0xFF  # make an empty page
 
             read_hex_page(file_state, page_addr, page_size, page_buffer)
 
@@ -122,7 +160,7 @@ class AVRprog:
                     print("skipping")
                 continue
 
-            #print("From HEX file: ", page_buffer)
+            # print("From HEX file: ", page_buffer)
             self._flash_page(bytearray(page_buffer), page_addr, page_size)
 
             if not verify:
@@ -134,20 +172,23 @@ class AVRprog:
                 print("Verifying page @ $%04X" % page_addr)
             read_buffer = bytearray(page_size)
             self.read(page_addr, read_buffer)
-            #print("From memory: ", read_buffer)
+            # print("From memory: ", read_buffer)
 
             if page_buffer != read_buffer:
                 if verbose:
                     # pylint: disable=line-too-long
-                    print("Verify fail at address %04X\nPage should be: %s\nBut contains: %s" % (page_addr, page_buffer, read_buffer))
+                    print(
+                        "Verify fail at address %04X\nPage should be: %s\nBut contains: %s"
+                        % (page_addr, page_buffer, read_buffer)
+                    )
                     # pylint: enable=line-too-long
                 self.end()
                 return False
 
-            if file_state['eof']:
-                break # we're done, bail!
+            if file_state["eof"]:
+                break  # we're done, bail!
 
-        file_state['f'].close()
+        file_state["f"].close()
         self.end()
         return True
 
@@ -160,15 +201,16 @@ class AVRprog:
             raise RuntimeError("Signature read failure")
 
         # create a file state dictionary
-        file_state = {'line': 0, 'ext_addr': 0, 'eof': False}
-        file_state['f'] = open(file_name, 'r')
+        file_state = {"line": 0, "ext_addr": 0, "eof": False}
+        file_state["f"] = open(file_name, "r")
 
-        page_size = chip['page_size']
-        self.begin()
-        for page_addr in range(0x0, chip['flash_size'], page_size):
+        page_size = chip["page_size"]
+        clock_speed = getattr(chip, "clock_speed", _FAST_CLOCK)
+        self.begin(clock=clock_speed)
+        for page_addr in range(0x0, chip["flash_size"], page_size):
             page_buffer = bytearray(page_size)
             for b in range(page_size):
-                page_buffer[b] = 0xFF     # make an empty page
+                page_buffer[b] = 0xFF  # make an empty page
 
             read_hex_page(file_state, page_addr, page_size, page_buffer)
 
@@ -176,21 +218,24 @@ class AVRprog:
                 print("Verifying page @ $%04X" % page_addr)
             read_buffer = bytearray(page_size)
             self.read(page_addr, read_buffer)
-            #print("From memory: ", read_buffer)
-            #print("From file  : ", page_buffer)
+            # print("From memory: ", read_buffer)
+            # print("From file  : ", page_buffer)
 
             if page_buffer != read_buffer:
                 if verbose:
                     # pylint: disable=line-too-long
-                    print("Verify fail at address %04X\nPage should be: %s\nBut contains: %s" % (page_addr, page_buffer, read_buffer))
+                    print(
+                        "Verify fail at address %04X\nPage should be: %s\nBut contains: %s"
+                        % (page_addr, page_buffer, read_buffer)
+                    )
                     # pylint: enable=line-too-long
                 self.end()
                 return False
 
-            if file_state['eof']:
-                break # we're done, bail!
+            if file_state["eof"]:
+                break  # we're done, bail!
 
-        file_state['f'].close()
+        file_state["f"].close()
         self.end()
         return True
 
@@ -199,7 +244,7 @@ class AVRprog:
         Read the 4 fuses and return them in a list (low, high, ext, lock)
         Each fuse is bitwise-&'s with the chip's fuse mask for simplicity
         """
-        mask = chip['fuse_mask']
+        mask = chip["fuse_mask"]
         self.begin(clock=_SLOW_CLOCK)
         low = self._transaction((0x50, 0, 0, 0))[2] & mask[0]
         high = self._transaction((0x58, 0x08, 0, 0))[2] & mask[1]
@@ -216,10 +261,11 @@ class AVRprog:
         """
         self.begin(clock=_SLOW_CLOCK)
         lock and self._transaction((0xAC, 0xE0, 0, lock))
-        low  and self._transaction((0xAC, 0xA0, 0, low))
+        low and self._transaction((0xAC, 0xA0, 0, low))
         high and self._transaction((0xAC, 0xA8, 0, high))
-        ext  and self._transaction((0xAC, 0xA4, 0, ext))
+        ext and self._transaction((0xAC, 0xA4, 0, ext))
         self.end()
+
     # pylint: enable=unused-argument,expression-not-assigned
 
     def verify_fuses(self, chip, low=None, high=None, ext=None, lock=None):
@@ -237,7 +283,6 @@ class AVRprog:
                 return False
         return True
 
-
     def erase_chip(self):
         """
         Fully erases the chip.
@@ -286,18 +331,18 @@ class AVRprog:
         Requires calling begin() beforehand to put in programming mode.
         """
         last_addr = 0
-        for i in range(len(read_buffer)//2):
-            read_addr = addr//2 + i # read 'words' so address is half
+        for i in range(len(read_buffer) // 2):
+            read_addr = addr // 2 + i  # read 'words' so address is half
 
             if (last_addr >> 16) != (read_addr >> 16):
                 # load extended byte
-                #print("Loading extended address",  read_addr >> 16)
+                # print("Loading extended address",  read_addr >> 16)
                 self._transaction((0x4D, 0, read_addr >> 16, 0))
             high = self._transaction((0x28, read_addr >> 8, read_addr, 0))[2]
             low = self._transaction((0x20, read_addr >> 8, read_addr, 0))[2]
-            #print("%04X: %02X %02X" % (read_addr*2, low, high))
-            read_buffer[i*2] = low
-            read_buffer[i*2+1] = high
+            # print("%04X: %02X %02X" % (read_addr*2, low, high))
+            read_buffer[i * 2] = low
+            read_buffer[i * 2 + 1] = high
 
             last_addr = read_addr
 
@@ -307,9 +352,9 @@ class AVRprog:
         self._transaction((0x48, addr >> 8, addr, high))
 
     def _flash_page(self, page_buffer, page_addr, page_size):
-        page_addr //= 2 # address is by 'words' not bytes!
-        for i in range(page_size/2): # page indexed by words, not bytes
-            lo_byte, hi_byte = page_buffer[2*i:2*i+2]
+        page_addr //= 2  # address is by 'words' not bytes!
+        for i in range(page_size / 2):  # page indexed by words, not bytes
+            lo_byte, hi_byte = page_buffer[2 * i : 2 * i + 2]
             self._flash_word(i, lo_byte, hi_byte)
 
         # load extended byte
@@ -325,16 +370,17 @@ class AVRprog:
         command = bytearray([i & 0xFF for i in command])
 
         self._spi.write_readinto(command, reply)
-        #s = [hex(i) for i in command]
-        #print("Sending %s reply %s" % ([hex(i) for i in command], [hex(i) for i in reply]))
+        # s = [hex(i) for i in command]
+        # print("Sending %s reply %s" % ([hex(i) for i in command], [hex(i) for i in reply]))
         if reply[2] != command[1]:
             raise RuntimeError("SPI transaction failed")
-        return reply[1:] # first byte is ignored
+        return reply[1:]  # first byte is ignored
 
     def _busy_wait(self):
         while self._transaction((0xF0, 0, 0, 0))[2] & 0x01:
             pass
 
+
 def read_hex_page(file_state, page_addr, page_size, page_buffer):
     """
     Helper function that does the Intel Hex parsing. Takes in a dictionary
@@ -354,84 +400,91 @@ def read_hex_page(file_state, page_addr, page_size, page_buffer):
     we've done the best job we can with filling the buffer and the next
     line does not contain any more data we can use.
     """
-    while True:     # read until our page_buff is full!
-        orig_loc = file_state['f'].tell()  # in case we have to 'back up'
-        line = file_state['f'].readline()  # read one line from the HEX file
-        file_state['line'] += 1
+    while True:  # read until our page_buff is full!
+        orig_loc = file_state["f"].tell()  # in case we have to 'back up'
+        line = file_state["f"].readline()  # read one line from the HEX file
+        file_state["line"] += 1
 
         if not line:
-            file_state['eof'] = True
+            file_state["eof"] = True
             return False
-        #print(line)
-        if line[0] != ':':       # lines must start with ':'
-            raise RuntimeError("HEX line %d doesn't start with :" % file_state['line'])
+        # print(line)
+        if line[0] != ":":  # lines must start with ':'
+            raise RuntimeError("HEX line %d doesn't start with :" % file_state["line"])
 
         # Try to parse the line length, address, and record type
         try:
             hex_len = int(line[1:3], 16)
             line_addr = int(line[3:7], 16)
-            file_state['line_addr'] = line_addr
+            file_state["line_addr"] = line_addr
             rec_type = int(line[7:9], 16)
         except ValueError:
-            raise RuntimeError("Could not parse HEX line %d addr" % file_state['line'])
+            raise RuntimeError("Could not parse HEX line %d addr" % file_state["line"])
 
-        if file_state['ext_addr']:
-            line_addr += file_state['ext_addr']
-        #print("Hex len: %d, addr %04X, record type %d " % (hex_len, line_addr, rec_type))
+        if file_state["ext_addr"]:
+            line_addr += file_state["ext_addr"]
+        # print("Hex len: %d, addr %04X, record type %d " % (hex_len, line_addr, rec_type))
 
         # We should only look for data type records (0x00)
         if rec_type == 1:
-            file_state['eof'] = True
-            return False   # reached end of file
+            file_state["eof"] = True
+            return False  # reached end of file
         if rec_type == 2:
-            file_state['ext_addr'] = int(line[9:13], 16) << 4
-            #print("Extended addr: %05X" % file_state['ext_addr'])
+            file_state["ext_addr"] = int(line[9:13], 16) << 4
+            # print("Extended addr: %05X" % file_state['ext_addr'])
             continue
         if rec_type == 3:  # sometimes appears, we ignore this
             continue
-        elif rec_type != 0: # if not the above or a data record...
-            raise RuntimeError("Unsupported record type %d on line %d" %
-                               (rec_type, file_state['line']))
+        if rec_type != 0:  # if not the above or a data record...
+            raise RuntimeError(
+                "Unsupported record type %d on line %d" % (rec_type, file_state["line"])
+            )
 
         # check if this file file is either after the current page
         # (in which case, we've read all we can for this page and should
         # commence flasing...)
         if line_addr >= (page_addr + page_size):
-            #print("Hex is past page address range")
-            file_state['f'].seek(orig_loc)  # back up!
-            file_state['line'] -= 1
+            # print("Hex is past page address range")
+            file_state["f"].seek(orig_loc)  # back up!
+            file_state["line"] -= 1
             return True
         # or, this line does not yet reach the current page address, in which
         # case which should just keep reading in hopes we reach the address
         # we're looking for next time!
         if (line_addr + hex_len) <= page_addr:
-            #print("Hex is prior to page address range")
+            # print("Hex is prior to page address range")
             continue
 
         # parse out all remaining hex bytes including the checksum
         byte_buffer = []
         for i in range(hex_len + 1):
-            byte_buffer.append(int(line[9+i*2:11+i*2], 16))
+            byte_buffer.append(int(line[9 + i * 2 : 11 + i * 2], 16))
 
         # check chksum now!
-        chksum = hex_len + (line_addr >> 8) + (line_addr & 0xFF) + rec_type + sum(byte_buffer)
-        #print("checksum: "+hex(chksum))
+        chksum = (
+            hex_len
+            + (line_addr >> 8)
+            + (line_addr & 0xFF)
+            + rec_type
+            + sum(byte_buffer)
+        )
+        # print("checksum: "+hex(chksum))
         if (chksum & 0xFF) != 0:
             raise RuntimeError("HEX Checksum fail")
 
         # get rid of that checksum byte
         byte_buffer.pop()
-        #print([hex(i) for i in byte_buffer])
+        # print([hex(i) for i in byte_buffer])
 
-        #print("line addr $%04X page addr $%04X" % (line_addr, page_addr))
+        # print("line addr $%04X page addr $%04X" % (line_addr, page_addr))
         page_idx = line_addr - page_addr
         line_idx = 0
         while (page_idx < page_size) and (line_idx < hex_len):
-            #print("page_idx = %d, line_idx = %d" % (page_idx, line_idx))
+            # print("page_idx = %d, line_idx = %d" % (page_idx, line_idx))
             page_buffer[page_idx] = byte_buffer[line_idx]
             line_idx += 1
             page_idx += 1
         if page_idx == page_size:
-            return True # ok we've read a full page, can bail now!
+            return True  # ok we've read a full page, can bail now!
 
-    return False # we...shouldn't get here?
+    return False  # we...shouldn't get here?
